/*
LinuxCNC component for controlling the MAHO MH400E gearbox.

Copyright (C) 2018 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef __MH400E_COMMON_H__
#define __MH400E_COMMON_H__

/* structure that allows to group pins together */
#define MH400E_PINS_IN_GROUP    3
typedef struct
{
    hal_bit_t *p[MH400E_PINS_IN_GROUP];
} pin_group_t;

/* description of a particular gear/speed setting */
typedef struct
{
    unsigned key;
    unsigned value;
} pair_t;


/* Each shaft is described by 3 bits:
 *          2       1      0
 * msb  "center" "right" "left" lsb
 *
 * The table below provides the mask for all three shafts.
 * The helper macros should be used on an extracted 3 bit value.
 */
#define MH400E_STAGE_IS_LEFT(mask)          (mask & 1)
#define MH400E_STAGE_IS_RIGHT(mask)         ((mask >> 1) & 1)
#define MH400E_STAGE_IS_CENTER(mask)        ((mask >> 2) & 1)



/* lookup table from rpm to gearbox status pin values */
static pair_t mh400e_gears[] =
{  /* rpm   bitmask         msb 9   8   7   6   5   4   3   2   1  lsb */
						/*		Block 1     Block 2	    Vorgelege	*/
						/*		c   r   l   c   r   l   c   r   l	*/
	{	0,4	 },			/*								1	0	0	*/			
	{	100,162	},		/*		0	1	0	1	0	0	0	1	0	*/
	{	126,98	},		/*		0	0	1	1	0	0	0	1	0	*/
	{	160,274	},		/*		1	0	0	0	1	0	0	1	0	*/
	{	200,146 },		/*		0	1	0	0	1	0	0	1	0	*/
	{	250,82	},		/*		0	0	1	0	1	0	0	1	0	*/
	{	315,266	},		/*		1	0	0	0	0	1	0	1	0	*/
	{	400,138 },		/*		0	1	0	0	0	1	0	1	0	*/
	{	500,74	},		/*		0	0	1	0	0	1	0	1	0	*/
	{	630,289	},		/*		1	0	0	1	0	0	0	0	1	*/
	{	800,161 },		/*		0	1	0	1	0	0	0	0	1	*/
	{	1000,97 },		/*		0	0	1	1	0	0	0	0	1	*/
	{	1250,273},		/*		1	0	0	0	1	0	0	0	1	*/
	{	1600,145},		/*		0	1	0	0	1	0	0	0	1	*/
	{	2000,81 },		/*		0	0	1	0	1	0	0	0	1	*/
	{	2500,265},		/*		1	0	0	0	0	1	0	0	1	*/
	{	3150,137},		/*		0	1	0	0	0	1	0	0	1	*/
	{	4000,73 }		/*		0	0	1	0	0	1	0	0	1	*/	
};



#define  MH400E_STAGE_POS_LEFT              1   /* 100 */
/* Middle shaft position, marked as "bloue" on the MAHO */
#define MH400E_STAGE_POS_CENTER             4   /* 010 */
/* Furthest CW position, marked as "yellow" on the MAHO */
#define MH400E_STAGE_POS_RIGHT              2 	/* 001 */

/* total number of selectable gears including neutral */
#define MH400E_NUM_GEARS        (sizeof(mh400e_gears)/sizeof(pair_t))
/* max gear index in array */
#define MH400E_MAX_GEAR_INDEX       MH400E_NUM_GEARS  - 1
/* index of neutral gear */
#define MH400E_NEUTRAL_GEAR_INDEX   0
/* min spindle rpm > 0 supported by the MH400E */
#define MH400E_MIN_RPM              40
/* index of the first non 0 rpm setting in the gears array */
#define MH400E_MIN_RPM_INDEX        1
/* max spindle rpm supported by the MH400E */
#define MH400E_MAX_RPM              4000

#define MH400E_TWITCH_KEEP_PIN_ON   100*1000000L /* 800ms in nanoseconds */
#define MH400E_TWITCH_KEEP_PIN_OFF  500*1000000L /* 200ms in nanoseconds */

/* When shifting, poll the stage pins each 5ms. Picking a lower value here
 * to make sure that we do not "miss" and do not overshoot our target
 * position. */
#define MH400E_GEAR_STAGE_POLL_INTERVAL 5*1000000L /* 5ms in nanoseconds */

/* If reverse direction needs to be activated, we have to wait 100ms before
 * we activate the motor after the reverse pin has been activated or
 * deactivated */
#define MH400E_REVERSE_MOTOR_INTERVAL   100*1000000L /* 100ms in nanoseconds */

/* Interval between all remaining pin operations related to gear shifting */
#define MH400E_GENERIC_PIN_INTERVAL     100*1000000L /* 100ms in nanoseconds */

/* TODO: make this a module parameter */
#define MH400E_WAIT_SPINDLE_AT_SPEED    500*1000000L /* 500ms in nanoseconds */
/* generic state function */
typedef void (*statefunc)(long period);

#endif//__MH400E_COMMON_H__
